;
; ABSTRACO GRFICA
; Uma caneta sobre uma folha de papel;
;
; F. Nunes Ferreira
; fnf@fe.up.pt
; 21 de Novembro de 2000
;


; Incluso da biblioteca grfica da verso 53 do DrScheme
;

(require-library "graphics.ss" "graphics")

; Primitivas grficas disponveis
; na abstrao Caneta sobre uma folha de papel
; --------------------------------------------
;
; Trata-se de uma abstraco, baseada numa janela
; grfica em que uma caneta desenha e pinta pontos, rectas 
; e elipses, e tambm desenha texto.
;
; As primitivas grficas
; e respectivos parmetros so as seguintes:
;
; janela: largura, altura, titulo
;    Cria uma janela grfica.
;
; cor: indice (inteiro entre 0 e 15)
;    Para seleccionar a cor da caneta.
;
; move: ponto (lista composta por x y)
;    Sem desenhar, move a caneta para o ponto.
;
; move-rel: ponto definido relativo  posio da 
;           caneta (lista composta por dx dy)
;    Sem desenhar, move a caneta para o ponto.
;
; desenha: pontos (lista de pontos)
;    Desenha a poligonal, definida por uma 
;    sequncia de pontos.
;    No final, a caneta ficar sobre o ltimo 
;    ponto da sequncia.
;
; desenha-oval: dois-pontos (lista de dois pontos: o
;               vrtices superior-esquerdo e 
;               inferior-direito de um rectngulo).
;    Desenha oval inscrita no rectngulo.
;    O rectngulo  definido por uma lista de 2
;    vrtices, especificados pela seguinte ordem: 
;    superior-esquerdo e inferior-direiro
;    No final, a caneta mantm a posio de partida.
;
; pinta: pontos (lista de pontos)
;    Pinta o polgono, definido por uma 
;    sequncia de pontos.
;    No final, a caneta ficar sobre o ltimo 
;    ponto da sequncia.
;
; pinta-oval: dois-pontos (lista de dois pontos: o
;             vrtices superior-esquerdo e 
;             inferior-direito de um rectngulo).
;    Pinta oval inscrita no rectngulo.
;    O rectngulo  definido por uma lista de 2
;    vrtices, especificados pela seguinte ordem: 
;    superior-esquerdo e inferior-direiro.
;    No final, a caneta mantm a posio de partida.
; 
; ponto: ponto (lista composta por x y)
;    Pinta o ponto e, no final, a caneta ficar sobre 
;    esse ponto.
;
; desenha-rel: pontos (lista de pontos relativos  posio da
;              caneta. Como a caneta se vai deslocando, cada 
;              ponto, excepto o primeiro,  definido 
;              relativamente ao ponto anterior.
;    Desenha a poligonal, definida pela 
;    sequncia de pontos.
;    No final, a caneta ficar sobre o ltimo 
;    ponto da sequncia.
;
; desenha-oval-rel: dois-pontos (lista de dois pontos, ambos
;                   definidos relativamente  posio da 
;                   caneta: o vrtice superior-esquerdo e 
;                   inferior-direito de um rectngulo).
;       Desenha oval inscrita no rectngulo.
;       O rectngulo  definido por uma lista de 2
;       vrtices, especificados pela seguinte ordem: 
;       superior-esquerdo e inferior-direiro.
;       No final, a caneta retoma a posio 
;       que tinha  partida. 
;
; pinta-rel:  pontos (lista de pontos relativos  posio da
;             caneta. Como a caneta se vai deslocando, cada 
;             ponto, excepto o primeiro,  definido 
;             relativamente ao ponto anterior.
;    Pinta o polgono, definido pela sequncia de pontos.
;    No final, a caneta ficar sobre o ltimo ponto.
;
; ponto-rel: ponto definido relativo  posio da 
;            caneta (lista composta por dx dy)
;    Desenha ponto e deixa a caneta nesse ponto.
;
; desenha-txt: texto (cadeia de caracteres)
;    Desenha o texto especificado, a partir 
;    da posio da caneta.
;    A caneta fica na parte final do texto.
;
; caneta: no tem parmetros
;    Devolve a posio actual da caneta.
;
; limpa: no tem parmetros
;    Limpa a janela grfica.
;

(define posi (list 0 0))                ; posicao inicial da caneta
(define cor-caneta (make-rgb 0 0 0))    ; cor inicial da caneta
(define cor-temp? #f)                   ; cor temporaria?
(define vp 0)                           ; viewport
(define altura 0)                       ; altura do viewport
;
; cria uma janela grfica
;
(define janela               
  (lambda (larg alt titulo)
    (open-graphics)
    (set! altura alt)
    (let ((vport (open-viewport titulo larg alt)))
      (set! vp vport))))
;
;  para seleccionar a cor da caneta ...
;
(define cor      
  (lambda (indice)       ; indice = 'temp (passando duas vezes com esta cor
    ;                    ; repe a cor existente antes da 1 passagem
    (let ((tab-cores
	   '((0 0 0)(0 0 0.5)(0 0.5 0)(0 0.5 0.5)
	     (0.5 0 0)(0.5 0 0.5)(0.5 0.5 0)(0.5 0.5 0.5)
	     (0.75 0.75 0.75)(0 0 1.0)(0 1.0 0)(0 1.0 1.0)
	     (1.0 0 0)(1.0 0 1.0)(1.0 1.0 0)(1.0 1.0 1.0))))
      (cond ((equal? indice 'temp)
             (set! cor-temp? #t))
            (else
	     (let ((cor-aux (list-ref tab-cores indice)))
	       (let ((cor-drscm (make-rgb (car cor-aux)
					  (cadr cor-aux)
					  (caddr cor-aux))))
		 (set! cor-temp? #f)
		 (set! cor-caneta cor-drscm))))))))
;
; Sem desenhar, move a caneta para um ponto
;
(define move             
  (lambda (ponto)        
    (set-car! posi (car ponto))
    (set-car! (cdr posi) (cadr ponto))))
;
; Sem desenhar, move a caneta para um ponto, o qual 
; definido relativo  posio corrente da caneta
;
(define move-rel        
  (lambda (d)            ; o parmetro d  a lista (dx dy)
    (set-car! posi (+ (car posi)
                      (car d)))
    (set-car! (cdr posi) (+ (cadr posi)
                            (cadr d)))))
;
; desenha uma poligonal, definida por uma sequncia de pontos
;
;      No final, a caneta ficar sobre o ltimo ponto da sequncia.
;
(define desenha         
  (lambda (seq-pontos)  
    (cond ((and (> (length seq-pontos) 1)
                cor-temp?)
           ((flip-line vp)       ; cor temporria
            (pt->posn (car seq-pontos) '(0 0))
            (pt->posn (cadr seq-pontos) '(0 0)))
           (desenha (cdr seq-pontos)))
          ((> (length seq-pontos) 1)
           ((draw-line vp)       ; cor escolhida
            (pt->posn (car seq-pontos) '(0 0))
            (pt->posn (cadr seq-pontos) '(0 0))
            cor-caneta)
           (desenha (cdr seq-pontos))))
    ;                                      
    ;  e deixa caneta no ltimo ponto
    ;
    (let ((ultimo (car (reverse seq-pontos))))  
      (set-car! posi (car ultimo))
      (set-car! (cdr posi) (cadr ultimo)))))
;
; desenha oval inscrita num rectngulo
;
; O rectngulo  definido por uma lista de 2 vrtices,
; especificados pela seguinte ordem: 
;             superior-esquerdo e inferior-direiro
;
;         No final, a caneta mantm a posio de partida.
;
(define desenha-oval          
  (lambda (dois-vertices)  
    (let* ((v1 (car dois-vertices))
	   (v2 (cadr dois-vertices))
	   (v1x (car v1))
	   (v1y (cadr v1))
	   (larg (- (car v2) (car v1)))
	   (alt (- (cadr v1) (cadr v2))))
      (if cor-temp?
          ((flip-ellipse vp) (pt->posn v1 '(0 0)) larg alt)  
          
          ((draw-ellipse vp) (pt->posn v1 '(0 0)) larg alt cor-caneta)))))    
;
; pinta um polgono, definida por uma sequncia de pontos.
; Se o ltimo ponto for diferente do primeiro, o polgono
;  automaticamente fechado.
;
;     No final, a caneta ficar sobre o ltimo ponto da sequncia.
;
(define pinta  
  (lambda (seq-pontos) 
    (if cor-temp?
	((flip-solid-polygon vp) 
	 (list->posn-list seq-pontos '(0 0))
	 (make-posn 0 0))
	((draw-solid-polygon vp) 
	 (list->posn-list seq-pontos '(0 0))
	 (make-posn 0 0)
	 cor-caneta))         
    ;                                      
    ;  e deixa caneta no ltimo ponto
    ;
    (let ((ultimo (car (reverse seq-pontos)))) 
      (set-car! posi (car ultimo))
      (set-car! (cdr posi) (cadr ultimo)))))
;
; pinta oval inscrita num rectngulo
;
; O rectngulo  definido por uma lista de 2 vrtices,
; especificados pela seguinte ordem: 
;             superior-esquerdo e inferior-direiro
;
;         No final, a caneta mantm a posio de partida.
;
(define pinta-oval        
  (lambda (dois-vertices)  
    (let* ((v1 (car dois-vertices)) 
	   (v2 (cadr dois-vertices))
	   (v1x (car v1))
	   (v1y (cadr v1))
	   (larg (- (car v2) (car v1)))
	   (alt (- (cadr v1) (cadr v2))))
      (if cor-temp?
          ((flip-solid-ellipse vp) (pt->posn v1 '(0 0)) larg alt)  
          
          ((draw-solid-ellipse vp) (pt->posn v1 '(0 0)) larg alt cor-caneta)))))
;
; Como desenha, 
; mas os pontos da sequncia so definidos em modo relativo.
;
(define desenha-rel  
  (lambda (seq-pontos)      
    (let ((aux
	   (lambda (elem)
	     (begin 
	       (set-car! posi (+ (car elem)
				 (car posi)))
	       (set-car! (cdr posi) (+ (cadr elem)
				       (cadr posi)))
	       (list (car posi)
		     (cadr posi))))))
      
      (desenha (map aux 
                    seq-pontos)))))


;
; Como desenha-oval, 
; mas os vrtices so ambos definidos relativamente
;  posio corrente da caneta.
;
(define desenha-oval-rel          
  (lambda (dois-vertices)   
    (let ((corrente-x (car posi)) 
          (corrente-y (cadr posi))  
          (ver1 (car dois-vertices))
          (ver2 (cadr dois-vertices)))
      (desenha-oval (list
		     (list (+ corrente-x (car ver1))
			   (+ corrente-y (cadr ver1)))
		     (list (+ corrente-x (car ver2))
			   (+ corrente-y (cadr ver2))))))))

;
; Como pinta, 
; mas os pontos da sequncia so definidos em modo relativo.
;
(define pinta-rel 
  (lambda (seq-pontos)   
    (let ((aux
	   (lambda (elem)
	     (begin 
	       (set-car! posi (+ (car elem)
				 (car posi)))
	       (set-car! (cdr posi) (+ (cadr elem)
				       (cadr posi)))
	       (list (car posi)
		     (cadr posi))))))
      (pinta (map aux 
                  ;(cons '(0 0) seq-pontos))))))
                  seq-pontos)))))

;
; Como pinta-oval, 
; mas os vrtices so ambos definidos relativamente
;  posio corrente da caneta.
;
(define pinta-oval-rel         
  (lambda (dois-vertices)  
    (let ((corrente-x (car posi))  ; ao ponto corrente
          (corrente-y (cadr posi))  ; e ponto corrente mantm-se
          (ver1 (car dois-vertices))
          (ver2 (cadr dois-vertices)))
      (pinta-oval (list
                   (list (+ corrente-x (car ver1))
                         (+ corrente-y (cadr ver1)))
                   (list (+ corrente-x (car ver2))
                         (+ corrente-y (cadr ver2))))))))
;
; Desenha um ponto
;     No final, a caneta ficar sobre o ponto.
;
(define ponto    ; desenha ponto p
  (lambda (p)   ; e deixa caneta em p
    (if cor-temp?
        ((flip-pixel vp) (make-posn (car p) 
                                    (- altura (cadr p))) cor-caneta)
        ((draw-pixel vp) (make-posn (car p) 
                                    (- altura (cadr p))) cor-caneta))
    ;
    (set-car! posi (car p))
    (set-car! (cdr posi) (cadr p))))

; Como ponto,
; mas a coordenada ponto  definida relativamente 
;  posio da caneta
;
(define ponto-rel  
  (lambda (p)       
    (ponto (list (+ (car p)
		    (car posi))
		 (+ (cadr p)
		    (cadr posi))))))
; desenha texto,
; a partir da posio corrente da caneta
;
(define desenha-txt 
  (lambda (texto)
    (if cor-temp?
        ((flip-string vp) (make-posn (car posi)
                                     (- altura
                                        (cadr posi)))
         texto)
        
        ((draw-string vp) (make-posn (car posi)
                                     (- altura
                                        (cadr posi)))
         texto cor-caneta))     
    ;
    ;  e deixa caneta na parte final do texto...
    ;
    (let ((larg-alt ((get-string-size vp) texto)))  
      (set-car! posi (+ (car larg-alt)
			(car posi))))))
;
; Devolve a posio actual da caneta
;
(define caneta 
  (lambda ()
    (list (car posi)
	  (cadr posi))))
;
; Limpa a janela grfica
;
(define limpa
  (lambda ()
    ((clear-viewport vp))))

;
; ------------------------------
;
; Procedimentos auxiliares
; 
;
; Converte uma lista de pontos numa lista de pontos
; do DrScheme...
;
(define list->posn-list
  (lambda (s-pontos pos)
    (let
	((x-posi (car pos))
	 (y-posi (cadr pos)))
      (let
	  ((aux
	    (lambda (elem)
	      (make-posn
	       (+ (car elem)
		  x-posi)
	       (- altura
		  (+ (cadr elem)
		     y-posi))))))
	(map aux s-pontos)))))
;
; Converte um ponto num ponto do DrScheme
;
(define pt->posn
  (lambda (pt pos)
    (let ((x-posi (car pos))
	  (y-posi (cadr pos)))
      (make-posn
       (+ (car pt)
          x-posi)
       (- altura
          (+ (cadr pt)
             y-posi))))))

;
; No DrScheme, Graphical Full Scheme (MrEd)
; a constante pi no est definida. Sendo assim:

(define pi 3.14159265358979)

; Fim do ficheiro


